const std = @import("std");

///
/// unique and immutable integer
id: usize,

/// summary text of this node
summary: []const u8,

///
pub fn init(alloc: std.mem.Allocator, id: usize, summary: []const u8) !@This() {
    return .{
        .id        = id,
        .summary   = try alloc.dupe(u8, summary),
    };
}
/// pass the same allocator as init() call
pub fn deinit(self: *@This(), alloc: std.mem.Allocator) void {
    alloc.free(self.summary);
}

test "serialize" {
    const alloc = std.testing.allocator;
    var   node  = try init(alloc, 0, "helloworld");
    defer node.deinit(alloc);

    var json = std.ArrayList(u8).init(alloc);
    defer json.deinit();
    try std.json.stringify(node, .{}, json.writer());

    try std.testing.expectEqualStrings(
        json.items,
        \\{"id":0,"summary":"helloworld"}
    );
}
